#!/usr/bin/perl -w

# Columbo.pl - kev AT bournemouthbynight DOT co DOT uk
# www.bournemouthbynight.co.uk
# This script performs horizontal scanning, automatically executing info
# gathering tools on open ports.
# Most vulnerability assessment tools perform vertical scanning; i.e. a large
# number of ports on a small number of hosts, attempting to gather informaion
# as they go.  Horizontal scanning, alternatively, involves a small number of
# ports across a large number of machines.  The benefit of horizontal scanning
# is that the tests are more targetted towards information gathering.
# LtColumbo is the GUI.
#
# options:
# -P protocol = tcp or udp
# -p port = port number for tcp or udp scans
# -I = don't ping first
# -n name = name for output files
# -c command = command to run against matching hosts (use ^ for host ip)
# -a address = address range in nmap friendly format
# -b = batchmode using config file (parallel tests)
# -f file = config for batchmode (defaults to $PWD/columbo.cfg)

# sub scan - this sub does the actual work
sub scan {
  local($protocol, $port, $name, $cmd, $addr, $noicmp) = @_;
  local $nmapswitch;
  local $icmpswitch;

  if ($protocol eq "tcp") {
    $nmapswitch = "-sS";
  } elsif ($protocol eq "udp") {
    $nmapswitch = "-sU";
  } else {
    $nmapswitch = "-sP";
  }

  if ($noicmp == 1) {
    $icmpswitch = "-P0";
  } else {
    $icmpswitch = "";
  }

  my $tokens;

  if ($nmapswitch eq "-sP") {
    print "nmap $nmapswitch $addr\n";
    open NMAPOUT, "nmap $nmapswitch $addr -oG - |" or die "Can't find nmap: $!";
    while (<NMAPOUT>) {
      if (/Status: Up/) {
        @tokens = split;
        $_ = $cmd;
        s/\^/$tokens[1]/g;
        print "$_ > $name.$tokens[1]\n";
        system "$_ > $name.$tokens[1] &";
      }
    }
  } else {
    print "nmap $nmapswitch -p $port $addr $icmpswitch\n";
    open NMAPOUT, "nmap $nmapswitch -p $port $addr $icmpswitch -oG - |" or die "Can't find nmap: $!";
    while (<NMAPOUT>) {
      if (/$port\/open/) {
        @tokens = split;
        $_ = $cmd;
        s/\^/$tokens[1]/g;
        print "$_ > $name.$tokens[1]\n";
        system "$_ > $name.$tokens[1] &";
      }
    }
  }
  return 0;
}

# actual program start

my $port = -1;
my $protocol = "";
my $name = "";
my $cmd = "";
my $addr = "";
my $configfile = "";
my $batchmode = 0;
my $noicmp = 0;

my $optctr = 0;

# sort the command line options
while ($ARGV[$optctr]) {
  if ($ARGV[$optctr] eq "-p") {
    $port = $ARGV[($optctr+1)];
  } elsif ($ARGV[$optctr] eq "-P") {
    $protocol = $ARGV[($optctr+1)];
  } elsif ($ARGV[$optctr] eq "-I") {
    $noicmp = 1;
    $optctr -= 1;
  } elsif ($ARGV[$optctr] eq "-n") {
    $name = $ARGV[($optctr+1)];
  } elsif ($ARGV[$optctr] eq "-c") {
    $cmd = $ARGV[($optctr+1)];
  } elsif ($ARGV[$optctr] eq "-a") {
    $addr = $ARGV[($optctr+1)];
  } elsif ($ARGV[$optctr] eq "-b") {
    $batchmode = 1;
    $optctr -= 1;
  } elsif ($ARGV[$optctr] eq "-f") {
    $configfile = $ARGV[($optctr+1)];
  }
  $optctr += 2;
}

# work out what to do and print usage if incompatible options set
if (
  ($batchmode == 0) and
  ($name ne "") and ($cmd ne "") and ($addr ne "") and
  (((($protocol eq "tcp") or ($protocol eq "udp")) and ($port != -1))
    or ($noicmp == 0))
  ) {
  scan($protocol, $port, $name, $cmd, $addr, $noicmp);
} elsif (($batchmode == 1) and ($addr ne "")) {
  if ($configfile eq "") {
    $configfile = "columbo.cfg";
  }

  my @command;

  open CONFIGFH, $configfile or die "Cannot open config file, $configfile: $!";
  while (<CONFIGFH>) {
    if ((!(/^#/)) and ($_ ne "\n")) {
      ($name, $protocol, $port, @command) = split;
      $cmd = join(' ', @command);
      if (($protocol ne "tcp") and ($protocol ne "udp")) {
        $noicmp_t = 0;
      } else {
        $noicmp_t = $noicmp;
      }
      if (fork == 0) {
        scan($protocol, $port, $name, $cmd, $addr, $noicmp_t);
        exit;
      }
    }
  }

} else {
print <<text_blk;
columbo.pl - Kev AT bournemouthbynight DOT co DOT uk
This tool performs horizontal scanning, automatically executing info gathering
tools on open ports.

Usage: columbo.pl [-P <tcp | udp>] [-p <port>] [-c <cmd>] -a <addresses>
         [-n <name>] [-I] [-b] [-f configfile]
-P specifies the protocol; no -P = ICMP
-p specifies the port for tcp and udp
-c specifies the command to run on matching hosts.  Use ^ for ip address
-a specifies the address range in nmap-friendly format
-n specifies the output file name prefix
-I specifies no ICMP (-P0)
-b specifies batchmode using a config file (parallel tests)
-f specifies the config file for batchmode, defaults to \$PWD/columbo.cfg

Par example,
#columbo.pl -P tcp -p 80 -c "./nikto.pl -host ^" -a 10.0.0.0/24 -n nikto
#columbo.pl -P udp -p 161 -c "snmpwalk -c public -v 1 ^ .1.3" -n snmp -a 10.0.0.0/24 -I
#columbo.pl -c "traceroute ^" -a 10.0.0.0/24 -n tr
#columbo.pl -b

Best of luck.

text_blk
exit;
}

